import "core-js/modules/es.array.find.js";
import "core-js/modules/es.array.flat-map.js";
import "core-js/modules/es.array.unscopables.flat-map.js";
import "core-js/modules/es.array.map.js";
import "core-js/modules/es.array.join.js";
import "core-js/modules/es.array.concat.js";
import "core-js/modules/es.regexp.exec.js";
import "core-js/modules/es.string.match.js";
import "core-js/modules/es.regexp.constructor.js";
import "core-js/modules/es.regexp.to-string.js";
import "core-js/modules/web.dom-collections.for-each.js";
import "core-js/modules/es.object.assign.js";
import "core-js/modules/es.array.filter.js";
import "core-js/modules/es.function.name.js";

/* eslint-disable global-require, no-console, no-underscore-dangle */

/* eslint-disable import/no-dynamic-require, @typescript-eslint/no-var-requires */
// Based on https://github.com/storybookjs/storybook/blob/master/addons/storyshots/storyshots-core/src/frameworks/configure.ts
import fs from 'fs';
import path from 'path';
import global from 'global';
import { yellow } from 'ansi-colors';
import registerRequireContextHook from 'babel-plugin-require-context-hook/register';
import plur from 'plur';
import reactElementToJSXString from 'react-element-to-jsx-string';
import { toRequireContext } from '@storybook/core-common';
import * as framework from '@storybook/react';
import { getOptions, isStoryFnWithArgs } from './utils';
registerRequireContextHook();
var storybook = framework;

var isFile = function isFile(file) {
  try {
    return fs.lstatSync(file).isFile();
  } catch (err) {
    return false;
  }
};

var extensions = ['ts', 'tsx', 'js', 'jsx'];

var resolveFile = function resolveFile(configDir, filenames) {
  return filenames.flatMap(function (filename) {
    return extensions.map(function (ext) {
      return path.join(configDir, "".concat(filename, ".").concat(ext));
    });
  }).find(isFile) || false;
};

var getPreviewFile = function getPreviewFile(configDir) {
  return resolveFile(configDir, ['preview', 'config']);
};

var getMainFile = function getMainFile(configDir) {
  return resolveFile(configDir, ['main']);
};

var getConfigPathParts = function getConfigPathParts(input) {
  var configDir = path.resolve(input);

  if (fs.lstatSync(configDir).isDirectory()) {
    var output = {};
    var preview = getPreviewFile(configDir);
    var main = getMainFile(configDir);

    if (preview) {
      output.preview = preview;
    }

    if (main) {
      var _require = require(main),
          _require$stories = _require.stories,
          stories = _require$stories === void 0 ? [] : _require$stories;

      output.stories = stories.map(function (pattern) {
        var _toRequireContext = toRequireContext(pattern),
            basePath = _toRequireContext.path,
            recursive = _toRequireContext.recursive,
            match = _toRequireContext.match;

        var regex = new RegExp(match);
        return global.__requireContext(configDir, basePath, recursive, regex);
      });
    }

    return output;
  }

  return {
    preview: configDir
  };
};

var configure = function configure(options) {
  var configDir = options.configDir;

  var _getConfigPathParts = getConfigPathParts(configDir),
      preview = _getConfigPathParts.preview,
      stories = _getConfigPathParts.stories;

  if (preview) {
    var _require2 = require(preview),
        parameters = _require2.parameters,
        decorators = _require2.decorators,
        globals = _require2.globals,
        globalTypes = _require2.globalTypes,
        argTypesEnhancers = _require2.argTypesEnhancers;

    if (decorators) {
      decorators.forEach(function (decorator) {
        return storybook.addDecorator(decorator);
      });
    }

    if (parameters || globals || globalTypes) {
      storybook.addParameters(Object.assign(Object.assign({}, parameters), {}, {
        globals: globals,
        globalTypes: globalTypes
      }));
    }

    if (argTypesEnhancers) {
      argTypesEnhancers.forEach(function (enhancer) {
        return storybook.addArgTypesEnhancer(enhancer);
      });
    }
  }

  if ((stories == null ? void 0 : stories.length) > 0) {
    storybook.configure(stories, false, false);
  }
};

export default (function (configDir, _ref) {
  var outFile = _ref.outFile;
  configure({
    configDir: configDir
  });
  var snippets = storybook.raw().map(function (_ref2) {
    var group = _ref2.kind,
        name = _ref2.name,
        getOriginal = _ref2.getOriginal,
        _ref2$parameters = _ref2.parameters;
    _ref2$parameters = _ref2$parameters === void 0 ? {} : _ref2$parameters;
    var playroom = _ref2$parameters.playroom;

    var _getOptions = getOptions(playroom),
        disable = _getOptions.disable,
        reactElementToJSXStringOptions = _getOptions.reactElementToJSXStringOptions;

    if (disable) {
      return undefined;
    }

    console.log("Generating ".concat(yellow([group, name].join('/')), " snippet..."));
    var storyFn = getOriginal();
    var element = isStoryFnWithArgs(storyFn) ? storyFn(storyFn.args) : storyFn();
    var code = reactElementToJSXString(element, reactElementToJSXStringOptions);
    return {
      group: group,
      name: name,
      code: code
    };
  }).filter(Boolean);
  fs.writeFileSync(path.resolve(process.cwd(), outFile), JSON.stringify(snippets, null, 2));
  console.log("\n".concat(snippets.length, " ").concat(plur('snippet', snippets.length), " generated."));
});